/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.codenarc.rule.basic

import org.codenarc.rule.AbstractRuleTestCase
import org.codenarc.rule.Rule
import org.junit.Test

/**
 * Tests for EmptyStaticInitializerRule
 *
 * @author Hamlet D'Arcy
 */
class EmptyStaticInitializerRuleTest extends AbstractRuleTestCase {

    @Test
    void testRuleProperties() {
        assert rule.priority == 2
        assert rule.name == 'EmptyStaticInitializer'
    }

    @Test
    void testSuccessScenario() {
        final SOURCE = '''
            class MyClass {
                static {
                    println 'statement'
                }
            }
            class MyClass2 {
            }        '''
        assertNoViolations(SOURCE)
    }

    @Test
    void testSingleViolation() {
        final SOURCE = '''
            class MyClass {
                static {
                }
            }        '''
        assertSingleViolation(SOURCE, 3, 'static {', 'The class MyClass has an empty static initializer. It is safe to delete it')
    }

    protected Rule createRule() {
        new EmptyStaticInitializerRule()
    }
}
